package com.mx.dla.dda.contrato.transaccion.terminos.dtos;

import com.mx.dla.global.dtos.BaseDTO;

public class TerminosAuxiliarDTO  extends BaseDTO{

	private static final long serialVersionUID = -1359217918616258507L;
	
	private Long	id;
	private String	nombre;
	private String	pDda;
	private String	pEstudio;
	private String	desde;
	private String	hasta;
	private String	fechaInicio;
	private String	fechaFin;
	
	private Long	idOpcionalA;
	private Long	idOpcionalB;
	private Long	idOpcionalC;
	private Integer valorA;
	
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public String getNombre() {
		return nombre;
	}
	public void setNombre(String nombre) {
		this.nombre = nombre;
	}
	public String getpDda() {
		return pDda;
	}
	public void setpDda(String pDda) {
		this.pDda = pDda;
	}
	public String getpEstudio() {
		return pEstudio;
	}
	public void setpEstudio(String pEstudio) {
		this.pEstudio = pEstudio;
	}
	public String getDesde() {
		return desde;
	}
	public void setDesde(String desde) {
		this.desde = desde;
	}
	public String getHasta() {
		return hasta;
	}
	public void setHasta(String hasta) {
		this.hasta = hasta;
	}
	public String getFechaInicio() {
		return fechaInicio;
	}
	public void setFechaInicio(String fechaInicio) {
		this.fechaInicio = fechaInicio;
	}
	public String getFechaFin() {
		return fechaFin;
	}
	public void setFechaFin(String fechaFin) {
		this.fechaFin = fechaFin;
	}
	public Long getIdOpcionalA() {
		return idOpcionalA;
	}
	public void setIdOpcionalA(Long idOpcionalA) {
		this.idOpcionalA = idOpcionalA;
	}
	public Long getIdOpcionalB() {
		return idOpcionalB;
	}
	public void setIdOpcionalB(Long idOpcionalB) {
		this.idOpcionalB = idOpcionalB;
	}
	public Integer getValorA() {
		return valorA;
	}
	public void setValorA(Integer valorA) {
		this.valorA = valorA;
	}
	public Long getIdOpcionalC() {
		return idOpcionalC;
	}
	public void setIdOpcionalC(Long idOpcionalC) {
		this.idOpcionalC = idOpcionalC;
	}
	
	



}
